%% Description : This script is used for generating the neutral stability curves
%% for the Non-Dimensional Mathieu equation (3.4) in manuscript by Floquet analysis following (Kumar and Tuckerman 1994).

%% Subharmonic(SH) case (\tilde{alpha} = 1/2)
clear;
close all;
clc;

mu=0;                   %% To set growth rate zero in [exp(\tilde{mu} + i*\tilde{alpha})\tilde{t}] term to get nuetral stability curves 
alpha = 1.0/2;          %% for sub harmonic case \tilde{alpha} = 1/2
B_sh = [];              %% Matrix for saving the eigen values which are actually the B values in A-B bender chart for SH case 
N = 20;                 %% Total number of fourier modes taken in expansion
for A = -2:0.1:25;      %% Defining range of A to be taken on x-axis :- A = [-2,25] 

    n = 0:1:N;

    for  i = 1:N+1
    X(i) = (A + (mu + 1i*(alpha + n(i)))^2);  %% Numerical Floquet decompositon - This is generalised L_{n} in the supplementary material eqn. 3.3 
    end

    %% Q matrix declaration and generation
    %% See structure of L & Q matrix in  (Kumar and Tuckerman 1994)

    Q = zeros(2*N+2,2*N+2);  %we start from 0 to N, so total N+1 Fourier modes. For each \zeta, there are two components, a real part and an imaginary part.

    Q(1,1) = 1;
    Q(2,2) = -1;  
    Q(1,3) = 1;
    Q(2,4) = 1;
    j = 1;
    i = 3;
    for z = 1:2*N

    Q(i,j) = 1;
    Q(i,j+4) = 1;
    i = i+1;
    j = j+1;
    if j>2*N-2
         Q(i,j) = 1;
         Q(i+1,j+1) = 1;
        break
    end
    end
    z = 1;

    %% L matrix declaration and generation
    L = zeros(2*N+2,2*N+2);         
    for i = 1:2:2*N+2           
       L(i,i) = X(z);
       L(i+1,i+1) = L(i,i);
       z = z+1;
    end

    %% Generalized eigen value problem solver in matlab
    % V store eigen vector and D stores eigen values as diagonal matrix

    [V,D] = eig(L,-Q);          %the minus sign in Q is because L\cdot\zeta = -B Q\cdot\zeta. The matrix D will contain the eigenvalues

    %% Consider real eigen values only:- Sorting and arranging in increasing order 
    tf = isreal(D);
    
    %if all eigenvalues are real
    if tf
        M = [];
        M = max(D);         %% convert diagonal matrix D into a vector
        M = sort(M);        %% Arrange the eigen values in increasing order
        M = M(M>0);         %% Keeps only the positive elements of M 
        y= length(M)
        G = NaN(1,N+1);     
        a = M;
        G = [a(1:y) G(y+1:end)];   %% keep the positive eigen values obtained till its length and  rest of the values as NaN to make dimension of B_sh matrix consistent. This padding is probably not necessary when eigenvalues are real 
        B_sh = [B_sh ; G];
    %% For complex eigen values :- Sorting and finding eigen values whose complex part is zero and taking real part arranging in increasing order 
    else
       E = [];
       for i =1:2*N+2
           for j=1:2*N+2
                if imag(D(i,j))==0
                    E = [E real(D(i,j))];
                    E = E(E>0);
                end
            end
       end
        E = sort(E);
        y= length(E)
        G = NaN(1,N+1);
        a = E;
        G = [a(1:y) G(y+1:end)];
        B_sh = [B_sh ; G];
    end
end

%% Plotting the nuetral stability curve for SH case

A = -2:0.1:25;
p = []; q =[];
for i = 1:length(A)
for j = 1:N+1 
    
    p = [p A(i)];
    q = [q B_sh(i,j)];
    
 end
end
plot(p,q,'k.');
axis([-2 25 0 50]);



hold on;

%% Harmonic(H) case (\tilde{alpha} = 0)


mu=0;
alpha = 0;              %% for sub harmonic case \tilde{alpha} = 0
B_h = [];               %% Matrix for saving the eigen values which are actually the B values in A-B bender chart for Harmonic case 
N = 20;

for A = -2:0.1:25;

    n = 0:1:N;

    for  i = 1:N+1
    X(i) = (A + (1i*(alpha + n(i)))^2);         %% Numerical Floquet decompositon - This is generalised L_{n} in the supplementary material eqn. 3.3 
    end
%% Q matrix declaration and generation
    
    Q = zeros(2*N+2,2*N+2);

    Q(1,1) = 0;
    Q(2,2) = 0;
    Q(1,3) = 2;
    Q(2,4) = 0;
    j = 1;
    i = 3;
    for z = 1:2*N
        Q(i,j) = 1;
        Q(i,j+4) = 1;
        i = i+1;
        j = j+1;
        if j>2*N-2
             Q(i,j) = 1;
             Q(i+1,j+1) = 1;
            break
        end
    end
    z = 1;
    %% L matrix declaration and generation
    L = zeros(2*N+2,2*N+2);

    for i = 1:2:2*N+2
       L(i,i) = real(X(z));
       L(i+1,i+1) = L(i,i);
       L(i+1,i) = imag(X(z));
       L(i,i+1) = -imag(X(z));
       z = z+1;
    end
    
     %% Generalized eigen value problem solver in matlab
     
    [V,D] = eig(L,-Q);
    
     %% Consider real eigen values only:- Sorting and arranging in increasing order 
    tf = isreal(D);
    if tf
        M = [];
        M = max(D);
        M = sort(M);
        M = M(M>0);
        ahat  = M;
        y= length(M)
        G = NaN(1,N+1);
        a = M;
        if (y >= N+1)
            a = a(1:N+1);
            G = a;
            B_h = [B_h ; G];
        else
        G = [a(1:y) G(y+1:end)];
        B_h = [B_h ; G];
        end

    else
%% For complex eigen values :- Sorting and finding eigen values whose complex part is zero and taking real part arranging in increasing order         
           E = [];
           for i =1:2*N+2
               for j=1:2*N+2
                    if imag(D(i,j))==0
                        E = [E real(D(i,j))];
                        E = E(E>0);
                    end
                end
           end
        E = sort(E);
         ahat  = E;
        y= length(E)
        G = NaN(1,N+1);
        a = E;
        G = [a(1:y) G(y+1:end)];
        B_h = [B_h ; G];


    end
end
%% Plotting the nuetral stability curve for Harmonic case
A = -2:0.1:25;
u = []; v =[];
for i = 1:length(A)
for j = 1:N+1  
    u = [u A(i)];
    v = [v B_h(i,j)]; 
end
end
plot(u,v,'k.');
axis([-2 25 0 50]);


%% 
set(gcf,'Position',[250 250 1000 750]);                
axis([-2 25 0 50]);      
set(gca,'FontSize',30);
xlabel('$\mathcal{A}$','Interpreter','latex','FontSize',35,'FontWeight','bold');
y = ylabel('$\mathcal{B}$','Interpreter','latex','FontSize',35,'FontWeight','bold','rotation',1);
set(y, 'position', get(y,'position')-[0.8,0.1,0]);
hold off;
set(gcf,'paperpositionmode','auto');
saveas(gcf,'./bender_orzag','png');
