%% This script file is used to find (k,h) given (A,B) using equation 3.10 in the manuscript. 
%% The parameters currently provided will generate the stability chart 6b in the paper.
clear all;
close all;
clc;
%% XY.mat is file which contains data generated from numerical Floquet. A matrix O is contained in XY.mat 
load AB; % load (A,B) data generated from numerical Floquet

%% Number of rows and columns of O matrix
[d,n] = size(O); 

k = [];
B = [];
h = [];
%% Parameters
T = 0.1;
rhoI = 1;
rhoO = 0.01;
Omega = 2*pi;
R0 = 0.5;
m = 2;
k = 0:0.001:20;
for i = 1:length(k) 
    
     coeff = (T/(R0^3));
     num = (k(i)*R0*((k(i)*R0)^2 +m^2 - 1));
     den1 = rhoI*(besseli(m,k(i)*R0)/(0.5*(besseli(m-1,k(i)*R0) + besseli(m+1,k(i)*R0))));
     den2 = rhoO*(besselk(m,k(i)*R0)/(-0.5*(besselk(m-1,k(i)*R0)+besselk(m+1,k(i)*R0))));
     
     A(i) = (coeff*(num/(den1-den2)))/Omega^2;

end
plot(k,A);
xlabel('$k$','Interpreter','latex','FontSize',15,'FontWeight','bold');
ylabel('$\mathcal{A}$','Interpreter','latex','FontSize',15,'FontWeight','bold','rotation',1);

%% start contain the index of the row corresponding to the value of A for k=0. This is valid only m > 0 .

k = [];
start = round(A(2),3)*1000 + 2;

for i=start:d
   
     coeff = (T/(R0^3));
     num = @(k) (k*R0*((k*R0)^2 +m^2 - 1));
     den1 = @(k) rhoI*(besseli(m,k*R0)/(0.5*(besseli(m-1,k*R0) + besseli(m+1,k*R0))));
     den2 = @(k) rhoO*(besselk(m,k*R0)/(-0.5*(besselk(m-1,k*R0)+besselk(m+1,k*R0))));
     fun = @(k) (coeff*(num(k)/(den1(k)-den2(k))))/Omega^2 - O(i,1);
    
     %% Guess range for k
    x0 = [0.001 30];
    options = optimset('TolFun',1e-15);
    sol = fzero(fun,x0,options)           %% root finding function to solve for k for each A.

    k = [k sol];    %% vector k for saving the wavenumber k obtained by fzero function

    B = O(i,2:n);   %% Takes out the B values for each A to calulate h further.
    
    den1 = rhoI*(besseli(m,k(end)*R0)/(0.5*(besseli(m-1,k(end)*R0) + besseli(m+1,k(end)*R0))));
    den2 = rhoO*(besselk(m,k(end)*R0)/(-0.5*(besselk(m-1,k(end)*R0)+besselk(m+1,k(end)*R0))));

    a = (B*2*Omega^2*(den1 - den2))/((rhoI - rhoO)*k(end));
    h = [h a];
end
    s = [];
    
%% Plot for h vs k....
for i = 1:length(k)
for j = 1:n-1
    
   s = [s k(i)];
 
 end
end

plot(s,h,'k.'); hold on;
axis([0 20 0 200]);

set(gcf,'Position',[250 250 1000 750]);
set(gca,'FontSize',20)

xlabel('$k$','Interpreter','latex','FontSize',30,'FontWeight','bold');
y=ylabel('$h$','Interpreter','latex','FontSize',30,'FontWeight','bold','rotation',1);
set(y, 'position', get(y,'position')-[0.8,0,0]);
set(gcf,'paperpositionmode','auto');
saveas(gcf,'./stability_chart','png');
